local addonName, ns = ...
local _G = _G
local addon = ns.addon

local repBarTemplate = addon.repBarTemplate
addon.playerRepBarTemplate = addon:newTemplate(repBarTemplate)
local playerRepBarTemplate = addon.playerRepBarTemplate
local setfenv = _G.setfenv
setfenv(1, playerRepBarTemplate)

local type, pairs = _G.type, _G.pairs
local pcall = _G.pcall
local assert = _G.assert

local min = _G.min

local AceConfigRegistry
do
    AceConfigRegistry = _G.LibStub:GetLibrary('AceConfigRegistry-3.0')
end

local GetFactionInfoByID = _G.GetFactionInfoByID

local REP_DATA  = ns.REP_DATA
local REP_NAMES = ns.REP_NAMES

playerRepBarTemplate.getEnable = nil
playerRepBarTemplate.setEnable = nil
playerRepBarTemplate.defaults.enable = nil

function playerRepBarTemplate:isEnabled()
    --if ns.debugOnEnDisable then
    --    addon:Debug(self:GetName().." in playerRepBarTemplate:isEnabled()")
    --end
    local parent = self.parent
    local parentOptions = parent:getOptions()
    local factionID = self:getFactionID()
    assert(type(factionID) == 'number')
    return parentOptions.factions[factionID]
end


function playerRepBarTemplate:getFactionID()
    return self.factionID
end

function playerRepBarTemplate:getFactionName()
    self.factionName = self.factionName or GetFactionInfoByID(self:getFactionID())
    return self.factionName
end

function playerRepBarTemplate:OnInitialize()
    if self.initialized then
        return
    end
    self.cache = {}
    local name = self:GetName()
    if ns.debugOnEnDisable then
        addon:Debug(name.." in playerRepBarTemplate:OnInitialize()")
    end

    -- Make optionstable usable
    self.optionsTable.name = self:getFactionName()
    self.optionsTable.handler = self
    assert(self.optionsTable.type)
    assert(type(self.defaults)=='table')
    addon.defaults.char[name] = self.defaults
    --AceConfig:RegisterOptionsTable(name, self.optionsTable)
    --self.optionsFrame = AceConfigDialog:AddToBlizOptions(name, name, addonName)
    self.initialized = true
end

local oldEnable = playerRepBarTemplate.OnEnable
function playerRepBarTemplate:OnEnable()
    if not self.initialized then
        self:OnInitialize()
    end
    oldEnable(self)

    local name = self:GetName()
    if ns.debugOnEnDisable then
        addon:Debug(name.." in playerRepBarTemplate:OnEnable()")
    end
    self:RegisterEvent('UPDATE_FACTION', 'Invalidate')
    self.parent.optionsTable.args[name] = self.optionsTable
    AceConfigRegistry:NotifyChange(self.parent:GetName())
end

local oldDisable = playerRepBarTemplate.OnDisable
function playerRepBarTemplate:OnDisable()
    oldDisable(self)

    local name = self:GetName()
    if ns.debugOnEnDisable then
        addon:Debug(name.." in playerRepBarTemplate:OnDisable()")
    end
    self.parent.optionsTable.args[name] = nil
    AceConfigRegistry:NotifyChange(self.parent:GetName())
end

function playerRepBarTemplate:Redraw()
    if not self:isEnabled() then
        return
    end
    local options = self:getOptions()
    local barCache = self.cache

    local currentGlobalXP = self:getCurrentXP()

    -- update max settings if necessary
    if options.targetXP and currentGlobalXP > options.targetXP then
       options.targetXP = currentGlobalXP
    end
    if options.startXP and currentGlobalXP < options.startXP then
        options.startXP = currentGlobalXP
    end

    local startGlobalXP  = options.startXP  or self.defaults.targetXP or self:getStartXPMin()
    local targetGlobalXP = options.targetXP or self.defaults.targetXP or self:getTargetXPMax()

    local currentGlobalXPRatio = (currentGlobalXP-startGlobalXP)/(targetGlobalXP-startGlobalXP)

    assert(barCache)
    assert(barCache.frame, "No frame in " .. self:GetName() )
    local frame = barCache.frame
    local right = frame:GetRight()
    local left = frame:GetLeft()
    if not (right and left) then
        self:Invalidate()
        return
    end
    local width = frame:GetRight() - frame:GetLeft()
    -- texture for curent xp
    self:DrawPrimaryBar(currentGlobalXPRatio)

    -- textures and numbers for levels
    self:drawNumbers()

    --update options table
    self:updateOptionsTable()
end

function playerRepBarTemplate:getCurrentXP()
    local factionId = self:getFactionID()
    if not factionId then return 0 end
    local isOk, name, description, standingID, barMin, barMax,
          barValue, atWarWith, canToggleAtWar, isHeader, isCollapsed,
          hasRep, isWatched, isChild
        = pcall(function() return GetFactionInfoByID(factionId) end)
    if not isOk then
        return 0
    end
    assert(barMax - barMin == REP_DATA[standingID], "barMin=" .. barMin .. " barMax=" .. barMax)
    local levelRep = self:calcXpForLevel(standingID)
    assert(levelRep == barMin)
    local sum = barValue
    --addon:Debug(name,standingID,levelRep, "value",barValue,"min", barMin,"max",barMax, sum)
    return sum
end

function playerRepBarTemplate:addNameToTooltip(tooltip)
    if ns.debugTooltip then
        addon:Debug(self:GetName().." in barTemplate:addNameToTooltip()")
    end
    local factionName = self:getFactionName()
    tooltip:AddLine(factionName)
end

function playerRepBarTemplate:fillTooltip(tooltip)
    if ns.debugTooltip then
        addon:Debug(self:GetName().." in playerRepBarTemplate:fillTooltip()")
    end
    local currentGlobalXP = self:getCurrentXP()
    local targetGlobalXP = self:getOptions().targetXP or self:getTargetXPMax()
    local startGlobalXP = self:getOptions().startXP or self:getCurrentXP()
    local deltaXP = targetGlobalXP-startGlobalXP
    local value = currentGlobalXP-startGlobalXP
    self:addNameToTooltip(tooltip)
    self:fillTooltipOnlyPrimary(tooltip, value, deltaXP)
end

--module.defaults.targetXP = module:calcXpForLevel(module:getTargetLevelMax())